@@ -210,7 +210,7 @@ int wolfSSL_init_memory_heap(WOLFSSL_HEAP* heap)
     XMEMCPY(heap->sizeList, wc_MemSz, sizeof(wc_MemSz));
     XMEMCPY(heap->distList, wc_Dist,  sizeof(wc_Dist));
 
-    if (InitMutex(&(heap->memory_mutex)) != 0) {
+    if (wc_InitMutex(&(heap->memory_mutex)) != 0) {
         WOLFSSL_MSG("Error creating heap memory mutex");
         return BAD_MUTEX_E;
     }
@@ -389,10 +389,19 @@ int wolfSSL_StaticBufferSz(byte* buffer, word32 sz, int flag)
 
     /* creating only IO buffers from memory passed in, max TLS is 16k */
     if (flag & WOLFMEM_IO_POOL || flag & WOLFMEM_IO_POOL_FIXED) {
-        ava = sz % (memSz + padSz + WOLFMEM_IO_SZ);
+        if (ava < (memSz + padSz + WOLFMEM_IO_SZ)) {
+            return 0; /* not enough room for even one bucket */
+        }
+
+        ava = ava % (memSz + padSz + WOLFMEM_IO_SZ);
     }
     else {
         int i, k;
+
+        if (ava < (bucketSz[0] + padSz + memSz)) {
+            return 0; /* not enough room for even one bucket */
+        }
+
         while ((ava >= (bucketSz[0] + padSz + memSz)) && (ava > 0)) {
             /* start at largest and move to smaller buckets */
             for (i = (WOLFMEM_MAX_BUCKETS - 1); i >= 0; i--) {
@@ -524,7 +533,7 @@ void* wolfSSL_Malloc(size_t size, void* heap, int type)
         WOLFSSL_HEAP_HINT* hint = (WOLFSSL_HEAP_HINT*)heap;
         WOLFSSL_HEAP*      mem  = hint->memory;
 
-        if (LockMutex(&(mem->memory_mutex)) != 0) {
+        if (wc_LockMutex(&(mem->memory_mutex)) != 0) {
             WOLFSSL_MSG("Bad memory_mutex lock");
             return NULL;
         }
@@ -591,7 +600,7 @@ void* wolfSSL_Malloc(size_t size, void* heap, int type)
             WOLFSSL_MSG("ERROR ran out of static memory");
         }
 
-        UnLockMutex(&(mem->memory_mutex));
+        wc_UnLockMutex(&(mem->memory_mutex));
     }
 
     #ifdef WOLFSSL_MALLOC_CHECK
@@ -651,7 +660,7 @@ void wolfSSL_Free(void *ptr, void* heap, int type)
 
             /* get memory struct and add it to available list */
             pt = (wc_Memory*)((byte*)ptr - sizeof(wc_Memory) - padSz);
-            if (LockMutex(&(mem->memory_mutex)) != 0) {
+            if (wc_LockMutex(&(mem->memory_mutex)) != 0) {
                 WOLFSSL_MSG("Bad memory_mutex lock");
                 return;
             }
@@ -703,7 +712,7 @@ void wolfSSL_Free(void *ptr, void* heap, int type)
                     stats->totalFr++;
                 }
             }
-            UnLockMutex(&(mem->memory_mutex));
+            wc_UnLockMutex(&(mem->memory_mutex));
         }
     }
 
@@ -745,7 +754,7 @@ void* wolfSSL_Realloc(void *ptr, size_t size, void* heap, int type)
         WOLFSSL_HEAP*      mem  = hint->memory;
         word32 padSz = -(int)sizeof(wc_Memory) & (WOLFSSL_STATIC_ALIGN - 1);
 
-        if (LockMutex(&(mem->memory_mutex)) != 0) {
+        if (wc_LockMutex(&(mem->memory_mutex)) != 0) {
             WOLFSSL_MSG("Bad memory_mutex lock");
             return NULL;
         }
@@ -786,19 +795,19 @@ void* wolfSSL_Realloc(void *ptr, size_t size, void* heap, int type)
                 mem->alloc += 1;
 
                 /* free memory that was previously being used */
-                UnLockMutex(&(mem->memory_mutex));
+                wc_UnLockMutex(&(mem->memory_mutex));
                 wolfSSL_Free(ptr, heap, type
             #ifdef WOLFSSL_DEBUG_MEMORY
                     , func, line
             #endif
                 );
-                if (LockMutex(&(mem->memory_mutex)) != 0) {
+                if (wc_LockMutex(&(mem->memory_mutex)) != 0) {
                     WOLFSSL_MSG("Bad memory_mutex lock");
                     return NULL;
                 }
             }
         }
-        UnLockMutex(&(mem->memory_mutex));
+        wc_UnLockMutex(&(mem->memory_mutex));
     }
 
     #ifdef WOLFSSL_MALLOC_CHECK
