@@ -371,6 +371,11 @@ static int RsaMGF(int type, byte* seed, word32 seedSz, byte* out,
             break;
     #endif
     #ifndef NO_SHA256
+    #ifdef WOLFSSL_SHA224
+        case WC_MGF1SHA224:
+            ret = RsaMGF1(WC_HASH_TYPE_SHA224, seed, seedSz, out, outSz, heap);
+            break;
+    #endif
         case WC_MGF1SHA256:
             ret = RsaMGF1(WC_HASH_TYPE_SHA256, seed, seedSz, out, outSz, heap);
             break;
@@ -702,7 +707,7 @@ static int RsaUnPad_OAEP(byte *pkcsBlock, unsigned int pkcsBlockLen,
     /* find seedMask value */
     if ((ret = RsaMGF(mgf, (byte*)(pkcsBlock + (hLen + 1)),
                             pkcsBlockLen - hLen - 1, tmp, hLen, heap)) != 0) {
-        XFREE(tmp, NULL, DYNAMIC_TYPE_TMP_BUFFER);
+        XFREE(tmp, heap, DYNAMIC_TYPE_TMP_BUFFER);
         return ret;
     }
 
@@ -841,72 +846,6 @@ static int wc_RsaUnPad_ex(byte* pkcsBlock, word32 pkcsBlockLen, byte** out,
     return ret;
 }
 
-
-#ifdef WC_RSA_BLINDING
-
-/* helper for either lib */
-static int get_digit_count(mp_int* a)
-{
-    if (a == NULL)
-        return 0;
-
-    return a->used;
-}
-
-
-static int get_rand_digit(WC_RNG* rng, mp_digit* d)
-{
-    return wc_RNG_GenerateBlock(rng, (byte*)d, sizeof(mp_digit));
-}
-
-
-static int mp_rand(mp_int* a, int digits, WC_RNG* rng)
-{
-    int ret;
-    mp_digit d;
-
-    if (rng == NULL)
-        return MISSING_RNG_E;
-
-    if (a == NULL)
-        return BAD_FUNC_ARG;
-
-    mp_zero(a);
-    if (digits <= 0) {
-        return MP_OKAY;
-    }
-
-    /* first place a random non-zero digit */
-    do {
-        ret = get_rand_digit(rng, &d);
-        if (ret != 0) {
-            return ret;
-        }
-    } while (d == 0);
-
-    if ((ret = mp_add_d(a, d, a)) != MP_OKAY) {
-        return ret;
-    }
-
-    while (--digits > 0) {
-        if ((ret = mp_lshd(a, 1)) != MP_OKAY) {
-            return ret;
-        }
-        if ((ret = get_rand_digit(rng, &d)) != 0) {
-            return ret;
-        }
-        if ((ret = mp_add_d(a, d, a)) != MP_OKAY) {
-            return ret;
-        }
-    }
-
-    return ret;
-}
-
-
-#endif /* WC_RSA_BLINGING */
-
-
 static int wc_RsaFunctionSync(const byte* in, word32 inLen, byte* out,
                           word32* outLen, int type, RsaKey* key, WC_RNG* rng)
 {
@@ -961,10 +900,17 @@ static int wc_RsaFunctionSync(const byte* in, word32 inLen, byte* out,
         if (mp_exptmod(&tmp, &key->d, &key->n, &tmp) != MP_OKAY)
             ERROR_OUT(MP_EXPTMOD_E);
     #else
-        #define INNER_ERROR_OUT(x) { ret = (x); goto inner_done; }
+        /* Return 0 when cond is false and n when cond is true. */
+        #define COND_N(cond, n)    ((0 - (cond)) & (n))
+        /* If ret has an error value return it otherwise if r is OK then return
+         * 0 otherwise return e.
+         */
+        #define RET_ERR(ret, r, e) \
+            ((ret) | (COND_N((ret) == 0, COND_N((r) != MP_OKAY, (e)))))
     
         { /* tmpa/b scope */
         mp_int tmpa, tmpb;
+        int r;
 
         if (mp_init(&tmpa) != MP_OKAY)
             ERROR_OUT(MP_INIT_E);
@@ -975,35 +921,35 @@ static int wc_RsaFunctionSync(const byte* in, word32 inLen, byte* out,
         }
 
         /* tmpa = tmp^dP mod p */
-        if (mp_exptmod(&tmp, &key->dP, &key->p, &tmpa) != MP_OKAY)
-            INNER_ERROR_OUT(MP_EXPTMOD_E);
+        r = mp_exptmod(&tmp, &key->dP, &key->p, &tmpa);
+        ret = RET_ERR(ret, r, MP_EXPTMOD_E);
 
         /* tmpb = tmp^dQ mod q */
-        if (mp_exptmod(&tmp, &key->dQ, &key->q, &tmpb) != MP_OKAY)
-            INNER_ERROR_OUT(MP_EXPTMOD_E);
+        r = mp_exptmod(&tmp, &key->dQ, &key->q, &tmpb);
+        ret = RET_ERR(ret, r, MP_EXPTMOD_E);
 
         /* tmp = (tmpa - tmpb) * qInv (mod p) */
-        if (mp_sub(&tmpa, &tmpb, &tmp) != MP_OKAY)
-            INNER_ERROR_OUT(MP_SUB_E);
+        r = mp_sub(&tmpa, &tmpb, &tmp);
+        ret = RET_ERR(ret, r, MP_SUB_E);
 
-        if (mp_mulmod(&tmp, &key->u, &key->p, &tmp) != MP_OKAY)
-            INNER_ERROR_OUT(MP_MULMOD_E);
+        r = mp_mulmod(&tmp, &key->u, &key->p, &tmp);
+        ret = RET_ERR(ret, r, MP_MULMOD_E);
 
         /* tmp = tmpb + q * tmp */
-        if (mp_mul(&tmp, &key->q, &tmp) != MP_OKAY)
-            INNER_ERROR_OUT(MP_MUL_E);
+        r = mp_mul(&tmp, &key->q, &tmp);
+        ret = RET_ERR(ret, r, MP_MUL_E);
 
-        if (mp_add(&tmp, &tmpb, &tmp) != MP_OKAY)
-            INNER_ERROR_OUT(MP_ADD_E);
+        r = mp_add(&tmp, &tmpb, &tmp);
+        ret = RET_ERR(ret, r, MP_ADD_E);
 
-    inner_done:
         mp_clear(&tmpa);
         mp_clear(&tmpb);
 
         if (ret != 0) {
             goto done;
         }
-        #undef INNER_ERROR_OUT
+        #undef RET_ERR
+        #undef COND_N
         } /* tmpa/b scope */
     #endif   /* RSA_LOW_MEM */
 
@@ -1149,7 +1095,7 @@ int wc_RsaFunction(const byte* in, word32 inLen, byte* out,
    outLen: length of encrypted output buffer
    key   : wolfSSL initialized RSA key struct
    rng   : wolfSSL initialized random number struct
-   rsa_type  : type of RSA: RSA_PUBLIC_ENCRYPT, RSA_PUBLIC_DECRYPT, 
+   rsa_type  : type of RSA: RSA_PUBLIC_ENCRYPT, RSA_PUBLIC_DECRYPT,
         RSA_PRIVATE_ENCRYPT or RSA_PRIVATE_DECRYPT
    pad_value: RSA_BLOCK_TYPE_1 or RSA_BLOCK_TYPE_2
    pad_type  : type of padding: WC_RSA_PKCSV15_PAD or  WC_RSA_OAEP_PAD
@@ -1244,7 +1190,7 @@ static int RsaPublicEncryptEx(const byte* in, word32 inLen, byte* out,
    outLen: length of decrypted message in bytes
    outPtr: optional inline output pointer (if provided doing inline)
    key   : wolfSSL initialized RSA key struct
-   rsa_type  : type of RSA: RSA_PUBLIC_ENCRYPT, RSA_PUBLIC_DECRYPT, 
+   rsa_type  : type of RSA: RSA_PUBLIC_ENCRYPT, RSA_PUBLIC_DECRYPT,
         RSA_PRIVATE_ENCRYPT or RSA_PRIVATE_DECRYPT
    pad_value: RSA_BLOCK_TYPE_1 or RSA_BLOCK_TYPE_2
    pad_type  : type of padding: WC_RSA_PKCSV15_PAD or  WC_RSA_OAEP_PAD
@@ -1391,7 +1337,7 @@ int wc_RsaPrivateDecryptInline(byte* in, word32 inLen, byte** out, RsaKey* key)
 #ifdef WC_RSA_BLINDING
     rng = key->rng;
 #endif
-    return RsaPrivateDecryptEx(in, inLen, in, inLen, out, key, 
+    return RsaPrivateDecryptEx(in, inLen, in, inLen, out, key,
         RSA_PRIVATE_DECRYPT, RSA_BLOCK_TYPE_2, WC_RSA_PKCSV15_PAD,
         WC_HASH_TYPE_NONE, WC_MGF1NONE, NULL, 0, rng);
 }
@@ -1406,7 +1352,7 @@ int wc_RsaPrivateDecryptInline_ex(byte* in, word32 inLen, byte** out,
 #ifdef WC_RSA_BLINDING
     rng = key->rng;
 #endif
-    return RsaPrivateDecryptEx(in, inLen, in, inLen, out, key, 
+    return RsaPrivateDecryptEx(in, inLen, in, inLen, out, key,
         RSA_PRIVATE_DECRYPT, RSA_BLOCK_TYPE_2, type, hash,
         mgf, label, labelSz, rng);
 }
@@ -1449,7 +1395,7 @@ int wc_RsaSSL_VerifyInline(byte* in, word32 inLen, byte** out, RsaKey* key)
 #ifdef WC_RSA_BLINDING
     rng = key->rng;
 #endif
-    return RsaPrivateDecryptEx(in, inLen, in, inLen, out, key, 
+    return RsaPrivateDecryptEx(in, inLen, in, inLen, out, key,
         RSA_PUBLIC_DECRYPT, RSA_BLOCK_TYPE_1, WC_RSA_PKCSV15_PAD,
         WC_HASH_TYPE_NONE, WC_MGF1NONE, NULL, 0, rng);
 }
@@ -1461,7 +1407,7 @@ int wc_RsaSSL_Verify(const byte* in, word32 inLen, byte* out, word32 outLen,
 #ifdef WC_RSA_BLINDING
     rng = key->rng;
 #endif
-    return RsaPrivateDecryptEx((byte*)in, inLen, out, outLen, NULL, key, 
+    return RsaPrivateDecryptEx((byte*)in, inLen, out, outLen, NULL, key,
         RSA_PUBLIC_DECRYPT, RSA_BLOCK_TYPE_1, WC_RSA_PKCSV15_PAD,
         WC_HASH_TYPE_NONE, WC_MGF1NONE, NULL, 0, rng);
 }
@@ -1535,7 +1481,7 @@ int wc_MakeRsaKey(RsaKey* key, int size, long e, WC_RNG* rng)
     if ((err = mp_init_multi(&p, &q, &tmp1, &tmp2, &tmp3, NULL)) != MP_OKAY)
         return err;
 
-    err = mp_set_int(&tmp3, e);
+    err = mp_set_int(&tmp3, (mp_digit)e);
 
     /* make p */
     if (err == MP_OKAY) {
@@ -1577,7 +1523,7 @@ int wc_MakeRsaKey(RsaKey* key, int size, long e, WC_RNG* rng)
 
     /* make key */
     if (err == MP_OKAY)
-        err = mp_set_int(&key->e, e);  /* key->e = e */
+        err = mp_set_int(&key->e, (mp_digit)e);  /* key->e = e */
 
     if (err == MP_OKAY)                /* key->d = 1/e mod lcm(p-1, q-1) */
         err = mp_invmod(&key->e, &tmp1, &key->d);
@@ -1607,16 +1553,16 @@ int wc_MakeRsaKey(RsaKey* key, int size, long e, WC_RNG* rng)
         err = mp_copy(&q, &key->q);
 
     if (err == MP_OKAY)
-        key->type = RSA_PRIVATE; 
+        key->type = RSA_PRIVATE;
 
-    mp_clear(&tmp3); 
-    mp_clear(&tmp2); 
-    mp_clear(&tmp1); 
-    mp_clear(&q); 
+    mp_clear(&tmp3);
+    mp_clear(&tmp2);
+    mp_clear(&tmp1);
+    mp_clear(&q);
     mp_clear(&p);
 
     if (err != MP_OKAY) {
-        wc_FreeRsaKey(key);        
+        wc_FreeRsaKey(key);
         return err;
     }
 
