@@ -49,7 +49,6 @@ int wc_Sha256Final(Sha256* sha, byte* out)
     return Sha256Final_fips(sha, out);
 }
 
-
 #else /* else build without fips */
 
 #if !defined(NO_SHA256) && defined(WOLFSSL_TI_HASH)
@@ -82,11 +81,6 @@ int wc_Sha256Final(Sha256* sha, byte* out)
 #if defined(USE_INTEL_SPEEDUP)
 #define HAVE_INTEL_AVX1
 #define HAVE_INTEL_AVX2
-
-#if defined(DEBUG_XMM)
-#include "stdio.h"
-#endif
-
 #endif
 
 #if defined(HAVE_INTEL_AVX2)
@@ -219,7 +213,7 @@ static int set_cpuid_flags(void) {
 }
 
 
-/* #if defined(HAVE_INTEL_AVX1/2) at the tail of sha512 */
+/* #if defined(HAVE_INTEL_AVX1/2) at the tail of sha256 */
 static int Transform(Sha256* sha256);
 
 #if defined(HAVE_INTEL_AVX1)
@@ -251,7 +245,7 @@ static void set_Transform(void) {
 }
 
 #else
-   #if defined(FREESCALE_MMCAU)
+   #if defined(FREESCALE_MMCAU_SHA)
       #define XTRANSFORM(sha256, B) Transform(sha256, B)
    #else
       #define XTRANSFORM(sha256, B) Transform(sha256)
@@ -286,30 +280,27 @@ static void set_Transform(void) {
     #include <wolfcrypt/src/misc.c>
 #endif
 
-#ifdef FREESCALE_MMCAU
-    #include "cau_api.h"
+#ifdef FREESCALE_MMCAU_SHA
+    #include "fsl_mmcau.h"
 #endif
 
-#ifndef WOLFSSL_HAVE_MIN
-#define WOLFSSL_HAVE_MIN
-
-    static INLINE word32 min(word32 a, word32 b)
-    {
-        return a > b ? b : a;
-    }
-
-#endif /* WOLFSSL_HAVE_MIN */
-
 
+#ifdef FREESCALE_LTC_SHA
+int wc_InitSha256(Sha256* sha256)
+{
+    LTC_HASH_Init(LTC_BASE, &sha256->ctx, kLTC_Sha256, NULL, 0);
+    return 0;
+}
+#else
 int wc_InitSha256(Sha256* sha256)
 {
     int ret = 0;
-    #ifdef FREESCALE_MMCAU
+    #ifdef FREESCALE_MMCAU_SHA
         ret = wolfSSL_CryptHwMutexLock();
         if(ret != 0) {
             return ret;
         }
-        cau_sha256_initialize_output(sha256->digest);
+        MMCAU_SHA256_InitializeOutput((uint32_t*)sha256->digest);
         wolfSSL_CryptHwMutexUnLock();
     #else
         sha256->digest[0] = 0x6A09E667L;
@@ -332,9 +323,10 @@ int wc_InitSha256(Sha256* sha256)
 
     return ret;
 }
+#endif /* FREESCALE_LTC_SHA */
 
-
-#if !defined(FREESCALE_MMCAU)
+#if !defined(FREESCALE_LTC_SHA)
+#if !defined(FREESCALE_MMCAU_SHA)
 static const ALIGN32 word32 K[64] = {
     0x428A2F98L, 0x71374491L, 0xB5C0FBCFL, 0xE9B5DBA5L, 0x3956C25BL,
     0x59F111F1L, 0x923F82A4L, 0xAB1C5ED5L, 0xD807AA98L, 0x12835B01L,
@@ -353,19 +345,19 @@ static const ALIGN32 word32 K[64] = {
 
 #endif
 
-#if defined(FREESCALE_MMCAU)
+#if defined(FREESCALE_MMCAU_SHA)
 
 static int Transform(Sha256* sha256, byte* buf)
 {
     int ret = wolfSSL_CryptHwMutexLock();
     if(ret == 0) {
-        cau_sha256_hash_n(buf, 1, sha256->digest);
+        MMCAU_SHA256_HashN(buf, 1, (uint32_t*)sha256->digest);
         wolfSSL_CryptHwMutexUnLock();
     }
     return ret;
 }
 
-#endif /* FREESCALE_MMCAU */
+#endif /* FREESCALE_MMCAU_SHA */
 
 #define Ch(x,y,z)       ((z) ^ ((x) & ((y) ^ (z))))
 #define Maj(x,y,z)      ((((x) | (y)) & (z)) | ((x) & (y)))
@@ -383,7 +375,7 @@ static int Transform(Sha256* sha256, byte* buf)
      (d) += t0; \
      (h)  = t0 + t1;
 
-#if !defined(FREESCALE_MMCAU)
+#if !defined(FREESCALE_MMCAU_SHA)
 static int Transform(Sha256* sha256)
 {
     word32 S[8], t0, t1;
@@ -432,7 +424,7 @@ static int Transform(Sha256* sha256)
     return 0;
 }
 
-#endif /* #if !defined(FREESCALE_MMCAU) */
+#endif /* #if !defined(FREESCALE_MMCAU_SHA) */
 
 static INLINE void AddLength(Sha256* sha256, word32 len)
 {
@@ -440,8 +432,16 @@ static INLINE void AddLength(Sha256* sha256, word32 len)
     if ( (sha256->loLen += len) < tmp)
         sha256->hiLen++;                       /* carry low to high */
 }
+#endif /* FREESCALE_LTC_SHA */
 
+#ifdef FREESCALE_LTC_SHA
 int wc_Sha256Update(Sha256* sha256, const byte* data, word32 len)
+{
+    LTC_HASH_Update(&sha256->ctx, data, len);
+    return 0;
+}
+#else
+static INLINE int Sha256Update(Sha256* sha256, const byte* data, word32 len)
 {
 
     /* do block size increments */
@@ -460,7 +460,7 @@ int wc_Sha256Update(Sha256* sha256, const byte* data, word32 len)
         if (sha256->buffLen == SHA256_BLOCK_SIZE) {
             int ret;
 
-            #if defined(LITTLE_ENDIAN_ORDER) && !defined(FREESCALE_MMCAU)
+            #if defined(LITTLE_ENDIAN_ORDER) && !defined(FREESCALE_MMCAU_SHA)
                 #if defined(HAVE_INTEL_AVX1) || defined(HAVE_INTEL_AVX2)
                 if(!IS_INTEL_AVX1 && !IS_INTEL_AVX2)
                 #endif
@@ -479,7 +479,22 @@ int wc_Sha256Update(Sha256* sha256, const byte* data, word32 len)
     return 0;
 }
 
+int wc_Sha256Update(Sha256* sha256, const byte* data, word32 len)
+{
+    return Sha256Update(sha256, data, len);
+}
+
+#endif /* FREESCALE_LTC_SHA */
+
+#ifdef FREESCALE_LTC_SHA
 int wc_Sha256Final(Sha256* sha256, byte* hash)
+{
+    uint32_t hashlen = SHA256_DIGEST_SIZE;
+    LTC_HASH_Finish(&sha256->ctx, hash, &hashlen);
+    return wc_InitSha256(sha256);  /* reset state */
+}
+#else
+static INLINE int Sha256Final(Sha256* sha256)
 {
     byte* local = (byte*)sha256->buffer;
     int ret;
@@ -495,7 +510,7 @@ int wc_Sha256Final(Sha256* sha256, byte* hash)
         XMEMSET(&local[sha256->buffLen], 0, SHA256_BLOCK_SIZE - sha256->buffLen);
         sha256->buffLen += SHA256_BLOCK_SIZE - sha256->buffLen;
 
-        #if defined(LITTLE_ENDIAN_ORDER) && !defined(FREESCALE_MMCAU)
+        #if defined(LITTLE_ENDIAN_ORDER) && !defined(FREESCALE_MMCAU_SHA)
             #if defined(HAVE_INTEL_AVX1) || defined(HAVE_INTEL_AVX2)
             if(!IS_INTEL_AVX1 && !IS_INTEL_AVX2)
             #endif
@@ -516,7 +531,7 @@ int wc_Sha256Final(Sha256* sha256, byte* hash)
     sha256->loLen = sha256->loLen << 3;
 
     /* store lengths */
-    #if defined(LITTLE_ENDIAN_ORDER) && !defined(FREESCALE_MMCAU)
+    #if defined(LITTLE_ENDIAN_ORDER) && !defined(FREESCALE_MMCAU_SHA)
         #if defined(HAVE_INTEL_AVX1) || defined(HAVE_INTEL_AVX2)
         if(!IS_INTEL_AVX1 && !IS_INTEL_AVX2)
         #endif
@@ -527,7 +542,7 @@ int wc_Sha256Final(Sha256* sha256, byte* hash)
     XMEMCPY(&local[SHA256_PAD_SIZE + sizeof(word32)], &sha256->loLen,
             sizeof(word32));
 
-    #if defined(FREESCALE_MMCAU) || defined(HAVE_INTEL_AVX1) || defined(HAVE_INTEL_AVX2)
+    #if defined(FREESCALE_MMCAU_SHA) || defined(HAVE_INTEL_AVX1) || defined(HAVE_INTEL_AVX2)
         /* Kinetis requires only these bytes reversed */
         #if defined(HAVE_INTEL_AVX1) || defined(HAVE_INTEL_AVX2)
         if(IS_INTEL_AVX1 || IS_INTEL_AVX2)
@@ -537,7 +552,14 @@ int wc_Sha256Final(Sha256* sha256, byte* hash)
                          2 * sizeof(word32));
     #endif
 
-    ret = XTRANSFORM(sha256, local);
+    return XTRANSFORM(sha256, local);
+}
+
+int wc_Sha256Final(Sha256* sha256, byte* hash)
+{
+    int ret;
+
+    ret = Sha256Final(sha256);
     if (ret != 0)
         return ret;
 
@@ -548,7 +570,7 @@ int wc_Sha256Final(Sha256* sha256, byte* hash)
 
     return wc_InitSha256(sha256);  /* reset state */
 }
-
+#endif /* FREESCALE_LTC_SHA */
 
 
 
@@ -986,13 +1008,7 @@ __asm__ volatile("vmovdqu %0, %"#mask3 ::"m"(mSHUF_DC00[0])) ;
 
 static int Transform_AVX1(Sha256* sha256)
 {
-
-    word32 W_K[64] ;  /* temp for W+K */
-
-    #if defined(DEBUG_XMM)
-    int i, j ;
-    word32 xmm[29][4*15] ;
-    #endif
+    ALIGN32 word32 W_K[64] ;  /* temp for W+K */
 
     Init_Masks(BYTE_FLIP_MASK, SHUF_00BA, SHUF_DC00) ;
     W_K_from_buff ; /* X0, X1, X2, X3 = W[0..15] ; */
@@ -1063,16 +1079,6 @@ static int Transform_AVX1(Sha256* sha256)
         
     RegToDigest(S_0,S_1,S_2,S_3,S_4,S_5,S_6,S_7) ;  
         
-    #if defined(DEBUG_XMM)
-    for(i=0; i<29; i++) {
-        for(j=0; j<4*14; j+=4)
-            printf("xmm%d[%d]=%08x,%08x,%08x,%08x\n", j/4, i, 
-                   xmm[i][j],xmm[i][j+1],xmm[i][j+2],xmm[i][j+3]) ;
-        printf("\n") ;
-    }
-        
-    for(i=0; i<64; i++)printf("W_K[%d]%08x\n", i, W_K[i]) ;
-    #endif
 
     return 0;
 }
@@ -1080,13 +1086,7 @@ static int Transform_AVX1(Sha256* sha256)
 #if defined(HAVE_INTEL_RORX)
 static int Transform_AVX1_RORX(Sha256* sha256)
 {
-
-    word32 W_K[64] ;  /* temp for W+K */
-
-    #if defined(DEBUG_XMM)
-    int i, j ;
-    word32 xmm[29][4*15] ;
-    #endif
+    ALIGN32 word32 W_K[64] ;  /* temp for W+K */
 
     Init_Masks(BYTE_FLIP_MASK, SHUF_00BA, SHUF_DC00) ;
     W_K_from_buff ; /* X0, X1, X2, X3 = W[0..15] ; */
@@ -1156,16 +1156,6 @@ static int Transform_AVX1_RORX(Sha256* sha256)
         
     RegToDigest(S_0,S_1,S_2,S_3,S_4,S_5,S_6,S_7) ;  
         
-    #if defined(DEBUG_XMM)
-    for(i=0; i<29; i++) {
-        for(j=0; j<4*14; j+=4)
-            printf("xmm%d[%d]=%08x,%08x,%08x,%08x\n", j/4, i, 
-                    xmm[i][j],xmm[i][j+1],xmm[i][j+2],xmm[i][j+3]) ;
-        printf("\n") ;
-    }
-        
-    for(i=0; i<64; i++)printf("W_K[%d]%08x\n", i, W_K[i]) ;
-    #endif
 
     return 0;
 }
@@ -1724,15 +1714,59 @@ static int Transform_AVX2(Sha256* sha256)
 
     RegToDigest(S_0,S_1,S_2,S_3,S_4,S_5,S_6,S_7) ;  
 
-    #ifdef WOLFSSL_SMALL_STACK
-        XFREE(W, NULL, DYNAMIC_TYPE_TMP_BUFFER);
-    #endif
-    
+#ifdef WOLFSSL_SMALL_STACK
+    XFREE(W_K, NULL, DYNAMIC_TYPE_TMP_BUFFER);
+#endif
+
     return 0;
 }
 
 #endif   /* HAVE_INTEL_AVX2 */
 
+#ifdef WOLFSSL_SHA224
+int wc_InitSha224(Sha224* sha224)
+{
+    sha224->digest[0] = 0xc1059ed8;
+    sha224->digest[1] = 0x367cd507;
+    sha224->digest[2] = 0x3070dd17;
+    sha224->digest[3] = 0xf70e5939;
+    sha224->digest[4] = 0xffc00b31;
+    sha224->digest[5] = 0x68581511;
+    sha224->digest[6] = 0x64f98fa7;
+    sha224->digest[7] = 0xbefa4fa4;
+
+    sha224->buffLen = 0;
+    sha224->loLen   = 0;
+    sha224->hiLen   = 0;
+
+#if defined(HAVE_INTEL_AVX1)|| defined(HAVE_INTEL_AVX2)
+    set_Transform() ;
+#endif
+
+    return 0;
+}
+
+int wc_Sha224Update(Sha224* sha224, const byte* data, word32 len)
+{
+    return Sha256Update((Sha256 *)sha224, data, len);
+}
+
+
+int wc_Sha224Final(Sha224* sha224, byte* hash)
+{
+    int ret = Sha256Final((Sha256 *)sha224);
+    if (ret != 0)
+        return ret;
+
+    #if defined(LITTLE_ENDIAN_ORDER)
+        ByteReverseWords(sha224->digest, sha224->digest, SHA224_DIGEST_SIZE);
+    #endif
+    XMEMCPY(hash, sha224->digest, SHA224_DIGEST_SIZE);
+
+    return wc_InitSha224(sha224);  /* reset state */
+}
+#endif /* WOLFSSL_SHA224 */
+
 #endif   /* HAVE_FIPS */
 
 #endif   /* WOLFSSL_TI_HAHS */
