@@ -25,6 +25,19 @@
 
 #include <wolfssl/wolfcrypt/types.h>
 
+#ifndef NO_MD5
+    #include <wolfssl/wolfcrypt/md5.h>
+#endif
+#ifndef NO_SHA
+    #include <wolfssl/wolfcrypt/sha.h>
+#endif
+#if defined(WOLFSSL_SHA224) || !defined(NO_SHA256)
+    #include <wolfssl/wolfcrypt/sha256.h>
+#endif
+#if defined(WOLFSSL_SHA384) || defined(WOLFSSL_SHA512)
+    #include <wolfssl/wolfcrypt/sha512.h>
+#endif
+
 #ifdef __cplusplus
     extern "C" {
 #endif
@@ -36,12 +49,34 @@ enum wc_HashType {
     WC_HASH_TYPE_MD4 = 2,
     WC_HASH_TYPE_MD5 = 3,
     WC_HASH_TYPE_SHA = 4, /* SHA-1 (not old SHA-0) */
+    WC_HASH_TYPE_SHA224 = 9,
     WC_HASH_TYPE_SHA256 = 5,
     WC_HASH_TYPE_SHA384 = 6,
     WC_HASH_TYPE_SHA512 = 7,
     WC_HASH_TYPE_MD5_SHA = 8,
 };
 
+typedef union {
+    #ifndef NO_MD5
+        Md5 md5;
+    #endif
+    #ifndef NO_SHA
+        Sha sha;
+    #endif
+    #ifdef WOLFSSL_SHA224
+        Sha224 sha224;
+    #endif
+    #ifndef NO_SHA256
+        Sha256 sha256;
+    #endif
+    #ifdef WOLFSSL_SHA384
+        Sha384 sha384;
+    #endif
+    #ifdef WOLFSSL_SHA512
+        Sha512 sha512;
+    #endif
+} wc_HashAlg;
+
 /* Find largest possible digest size
    Note if this gets up to the size of 80 or over check smallstack build */
 #if defined(WOLFSSL_SHA512)
@@ -50,6 +85,8 @@ enum wc_HashType {
     #define WC_MAX_DIGEST_SIZE SHA384_DIGEST_SIZE
 #elif !defined(NO_SHA256)
     #define WC_MAX_DIGEST_SIZE SHA256_DIGEST_SIZE
+#elif defined(WOLFSSL_SHA224)
+    #define WC_MAX_DIGEST_SIZE SHA224_DIGEST_SIZE
 #elif !defined(NO_SHA)
     #define WC_MAX_DIGEST_SIZE SHA_DIGEST_SIZE
 #elif !defined(NO_MD5)
@@ -67,6 +104,13 @@ WOLFSSL_API int wc_Hash(enum wc_HashType hash_type,
     const byte* data, word32 data_len,
     byte* hash, word32 hash_len);
 
+/* generic hash operation wrappers */
+WOLFSSL_API int wc_HashInit(wc_HashAlg* hash, enum wc_HashType type);
+WOLFSSL_API int wc_HashUpdate(wc_HashAlg* hash, enum wc_HashType type,
+    const byte* data, word32 dataSz);
+WOLFSSL_API int wc_HashFinal(wc_HashAlg* hash, enum wc_HashType type,
+    byte* out);
+
 
 #ifndef NO_MD5
 #include <wolfssl/wolfcrypt/md5.h>
@@ -101,6 +145,12 @@ WOLFSSL_API int wc_Sha256Hash(const byte*, word32, byte*);
 #else
     #define wc_Sha256Free(d)
 #endif
+
+    #if defined(WOLFSSL_SHA224)
+        WOLFSSL_API int wc_Sha224GetHash(Sha224*, byte*);
+        WOLFSSL_API int wc_Sha224Hash(const byte*, word32, byte*);
+        #define wc_Sha224Free(d)
+    #endif /* defined(WOLFSSL_SHA224) */
 #endif
 
 #ifdef WOLFSSL_SHA512
